/****************************************************************************
**  MDHashTool - Message Digest (MD5, SHA-1, etc.) tool
**  Copyright (c) 2005  Charles Melhorn
**
**  This file is part of MDHashTool.
**
**  MDHashTool is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**  MDHashTool is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**
**  Or see the GNU web site at: http://www.gnu.org
**
****************************************************************************/ 

const nsIWebProgressListener = Components.interfaces.nsIWebProgressListener;

//const nsIMDHash = Components.interfaces.nsIMDHash;
//const gMDHashTool = Components.classes["@mozilla.org/mdhashtool;1"]
//                  .createInstance(nsIMDHash);


// Web progress listener for page load events. Used to add
// visual indicator to links with embedded checksum information.
// Algorithm suggested by TargetAlert extension (ver 0.7.4)
// written by Michael Bolin - http://www.bolinfest.com/

var mdhtWebProgListener = {

  QueryInterface : function(aIID)
  {
    if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
        aIID.equals(Components.interfaces.nsISupports))
      return this;
    throw Components.results.NS_NOINTERFACE;
  },

  onStateChange: function(aProgress, aRequest, aStateFlags, aStatus)
  {
    if ( (aStateFlags & nsIWebProgressListener.STATE_STOP) &&
         (aStateFlags & nsIWebProgressListener.STATE_IS_NETWORK ) ) // on page load
     {
       var doc = aProgress.DOMWindow.document;
       if ( (doc != null) && (gMDHTUserPreferenceInfo.getPrefValue("use_lf_icon")) )
        {
          var links = doc.links;  // all document links with href attribute (A and AREA elements)
          
          var lnk, cstype;
          for (var i=0; i < links.length; i++)
           {
             lnk = links[i];
             cstype = null;

             if ( (lnk.tagName.toLowerCase() == 'a') &&          // don't want AREA elements for now
                  (lnk.href.indexOf("#!") != -1) &&              // precheck for LF identifier
                  (cstype = isValidLinkFingerprint(lnk.href)) )
               mdhtAddIconToLink(doc, lnk, cstype[1]);
           }
        }
     }
  },
  onLocationChange:function(aWebProgress, aRequest, aLocation) {},
  onProgressChange:function(aProgress, aRequest, aCurSelfProgress, aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress){},
  onStatusChange:function(aWebProgress, aRequest, aStateFlags, aStatus){},
  onSecurityChange:function(aWebProgress, aRequest , aState){},
}


function mdhtAddIconToLink(doc, lnk, cstype)
{

  // check to see if icon has already been added to link
  // necessary for cases when STATE_STOP notification is
  // sent to web progress listener, but no page is loaded

  var childImgs;

  if (childImgs = lnk.getElementsByTagName('img'))
   {
     for (var i=0; i < childImgs.length; i++)
      {
        if (childImgs[i].getAttribute('src') == "chrome://mdhashtool/content/linkfprint22.gif")
          return;
      }
   }

  
  // append icon image to link
  var imageNode = doc.createElement("img");
  imageNode.setAttribute("src", "chrome://mdhashtool/content/linkfprint22.gif");
  imageNode.setAttribute("border", "0");
  imageNode.setAttribute("title", "[" + cstype + "] checksum-enhanced link");
  lnk.appendChild(imageNode);
}


// returns array resulting from match operation
// if valid, null if not

function isValidLinkFingerprint(href)
{
  var cs_array = null;
  var bValid = false;

  if ( cs_array = href.match(/#\!(md5|sha1)!([a-f0-9]+)/) )
   {
     // check hexvalue string length for corresponding hash type

     switch (cs_array[1])  
     {
       case "md5":   bValid = (cs_array[2].length == 32);
                     break;

       case "sha1":  bValid = (cs_array[2].length == 40);
                     break;

       default:      break;
     }
   }
  
  return (bValid ? cs_array : null);
 
}


/////////////////////////////////////////////////////////////////////
//
// Browser context menu items
//
/////////////////////////////////////////////////////////////////////

function mdhtOnContextMenuPopup()
{
  var bShowLFMenuItem = (gContextMenu.onSaveableLink &&   
                         isValidLinkFingerprint(gContextMenu.linkURL) )

  gContextMenu.showItem( "mdht-linkfp-menu", bShowLFMenuItem );   

}


function mdhtCopyLFChecksum()
{
  var cs_array = null;

  if (cs_array = isValidLinkFingerprint(gContextMenu.linkURL) )
   {
     var clipboardHelper = Components.classes["@mozilla.org/widget/clipboardhelper;1"]
                              .getService(Components.interfaces.nsIClipboardHelper);

     clipboardHelper.copyString(cs_array[2]);
   }  

}


function mdhtShowLFInfo()
{
  var cs_array = null;

  if (cs_array = isValidLinkFingerprint(gContextMenu.linkURL) )
   {
     var lfInfo = {};

     lfInfo.urlspec = gContextMenu.linkURL;
     lfInfo.urlspec = lfInfo.urlspec.substring(0, lfInfo.urlspec.indexOf("#!"));

     lfInfo.alg = cs_array[1];
     lfInfo.md = cs_array[2];

     window.openDialog('chrome://mdhashtool/content/dialogLFInfo.xul',
                       'mdhtLFInfo','chrome,centerscreen,modal', lfInfo);
   }

}


/////////////////////////////////////////////////////////////////////
//
// Automated Checksum Verification objects/methods
//
/////////////////////////////////////////////////////////////////////

/* 08/25/05 - download observer rewritten as a JS Component (nsMdhtAutoCS.js)

var mdhtDownloadObserver = {
  observe: function (aSubject, aTopic, aState) 
  {
    switch (aTopic)
    {
      case "dl-done":
        if (gMDHTUserPreferenceInfo.getPrefValue("auto_checksum"))
         {
           var dl = aSubject.QueryInterface(Components.interfaces.nsIDownload);
           var cs_array = null;

           if (cs_array = isValidLinkFingerprint(dl.source.spec) )
            {
              var hash_type;
              var reference_alg = cs_array[1].toLowerCase();
              var reference_digest = cs_array[2].toLowerCase();

              switch(reference_alg)
               {
                 case "md5":  hash_type = nsIMDHash.HASH_MD5;
                              break;

                 case "sha1": hash_type = nsIMDHash.HASH_SHA1;
                              break;

                 default:     hash_type = nsIMDHash.HASH_MD5;
                              break;
               }

              var checksum = gMDHashTool.getHashForFile(hash_type, dl.targetFile);

              if (reference_digest != checksum)
               {
                 var lfInfo = {};

                 lfInfo.urlspec = dl.source.spec;
                 lfInfo.urlspec = lfInfo.urlspec.substring(0, lfInfo.urlspec.indexOf("#!"));

                 lfInfo.alg = cs_array[1];
                 lfInfo.refdigest = cs_array[2];
                 lfInfo.compdigest = checksum;

                 window.openDialog('chrome://mdhashtool/content/dialogAutoCSAlert.xul',
                                   'mdhtACVAlert','chrome,centerscreen,modal', lfInfo);
               }

            }
         }

        break; 

      default:
        break;
    }
  }
};
*/


